"""
EagleBirth Python SDK Setup
"""

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="eaglebirth",
    version="1.1.3",
    author="EagleBirth",
    author_email="contact@eaglebirth.com",
    description="Official Python SDK for EagleBirth API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/eaglebirth/eaglebirth-python",
    project_urls={
        "Bug Tracker": "https://github.com/eaglebirth/eaglebirth-python/issues",
        "Documentation": "https://eaglebirth.com/developer/documentation",
        "Source Code": "https://github.com/eaglebirth/eaglebirth-python",
    },
    packages=find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests>=2.25.0",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "black>=22.0.0",
            "flake8>=4.0.0",
            "mypy>=0.950",
        ],
    },
)
